function DHM_final = DirectKinematics(table, joints, values, time)

%*************************************************
% Foreward kinematics for a robot following the David-Hartenberg rules
% Author: Rotem Sharoni
% Last updated 10th of June 2015
%*************************************************
%
%*************************************************************************
%
%  Input arguments:

%  table: a matrix that contains the parameters table of the Denavit-
% Hartenberg convention for a specific robot.

%  joints: a boolean row vector with the same length as the number of 
% joints that indicates the different types of joints. Revolute=1 and 
% Prismatic=0. Notice that this vector indicates the joint variable in each
% case.

%  values: a double precision matrix that contains the values for the joint
% variables (expressed in degrees, for revolute joints, or in meters, for 
% prismatic joints). The number of columns of this matrix is equal to the 
% number of joints and the number of rows corresponds to the different 
% time-steps that are to be evaluated.

%  time: a scalar that represents the time elapsed between two consecutive
% poses of the robot arm according to the matrix values.
%
%  Output arguments:
%  DHM_final: a 4x4 homogeneous matrix that represents the pose of the 
% end-effector w.r.t. the reference coordinate frame.
% The function must show (using different colors) every link, the rotation
% axes, the tool (using three lines to represent it), the reference 
% coordinate frame and the tool coordinate frame. The TCP 
% (Tool Center Point) must be joined to the X-Y plane by a vertical line.
% According to the arguments values and time, the robot must change its 
% pose.

%-----------------------------------
[ntimesteps, njoints] = size(values); %Gets number of timesteps and joints
figure;
%Get a DHM transformation matrix for each joint for each timestep
for t = 1:ntimesteps
    DHM_final(:,:,t) = eye(4); %Initialize the final matrix
    origin = [0 0 0]; %Initialize the origin
    % Delete the previous pose of the robot
    if t > 1
        delete(hand) %To only draw the next step
        delete(handax)
        delete(handay)
        delete(handaz)
        delete(tx);
        delete(ty);
        delete(tz);
    end
    for i = 1:njoints
        if joints(i) %Revolute
            DHM = denhartmat_final(values(t,i), table(i,2), table(i,3),...
                table(i,4));
        else %Prismatic
            DHM = denhartmat_final(table(i,1),values(t,i), table(i,3),...
                table(i,4));
        end %if joints(i)
            DHM_final(:,:,t) = DHM_final(:,:,t)*DHM; %T_0^i
            end_link = DHM_final(:,:,t)*[0 0 0 1]'; %End of link i-1
            hold on;
            view([10,10,10]);
            
        if i < njoints
        %Plots the links, and creates a handle for the plot    
        hand(i) = plot3([origin(1) end_link(1)], [origin(2)...
            end_link(2)],[origin(3) end_link(3)]); 
        else
        end_link = DHM_final(:,:,t)*[0 0 -table(3,2)/2 1]';%End of link i-1
        %Plots the links, and creates a handle for the plot 
        hand(i) = plot3([origin(1) end_link(1)], [origin(2)...
            end_link(2)],[origin(3) end_link(3)]); 
        end
            
            
        previous_frame = DHM_final(:,:,t)/(DHM);
        
        %Plot the reference coordinate frame
        x_axis = previous_frame*[0.15 0 0 1]';
        x_axis_plus = previous_frame*[0.20 0 0 1]'; %plus for the label
        tx(i)=text(x_axis_plus(1),x_axis_plus(2),x_axis_plus(3),...
            ['x',num2str(i-1)],'Color','r');
        y_axis = previous_frame*[0 0.15 0 1]';
        y_axis_plus = previous_frame*[0 0.20 0 1]';
        ty(i)=text(y_axis_plus(1),y_axis_plus(2),y_axis_plus(3),...
            ['y',num2str(i-1)],'Color','r');
        z_axis = previous_frame*[0 0 0.15 1]';
        z_axis_plus = previous_frame*[0 0 0.20 1]';
        tz(i)=text(z_axis_plus(1),z_axis_plus(2),z_axis_plus(3),...
            ['z',num2str(i-1)],'Color','r');
        handax(i) = plot3([origin(1) x_axis(1)], [origin(2) x_axis(2)],...
            [origin(3) x_axis(3)],'r');
        handay(i) = plot3([origin(1) y_axis(1)], [origin(2) y_axis(2)],...
            [origin(3) y_axis(3)],'r');
        handaz(i) = plot3([origin(1) z_axis(1)], [origin(2) z_axis(2)],...
            [origin(3) z_axis(3)],'r');

        %Recouperate the origin for the TCP point
        end_link = DHM_final(:,:,t)*[0 0 0 1]'; 
        if t > 1
            if not(i==njoints) %Keep the TCP to trace the trajectory
                delete(handlep(i)) %To only draw the next step
            end;
        end
        %Plots a point at every joint
        handlep(i) = plot3(end_link(1), end_link(2),end_link(3),'go'); 
        grid on;
        origin = end_link; %Update the "origin" to the end of the new link

%         %To draw the end effector
%         if i == njoints
% 
%         %Plot the reference coordinate frame for the end effector
% 
%             x_axis = DHM_final(:,:,t)*[0.15 0 0 1]';
%             x_axis_plus = DHM_final(:,:,t)*[0.20 0 0 1]';
%             tx(i+1)=text(x_axis_plus(1),x_axis_plus(2),x_axis_plus(3),...
%                 ['x',num2str(i)],'Color','r');
%             y_axis = DHM_final(:,:,t)*[0 0.15 0 1]';
%             y_axis_plus = DHM_final(:,:,t)*[0 0.20 0 1]';
%             ty(i+1)=text(y_axis_plus(1),y_axis_plus(2),y_axis_plus(3),...
%                 ['y',num2str(i)],'Color','r');
%             z_axis = DHM_final(:,:,t)*[0 0 0.15 1]';
%             z_axis_plus = DHM_final(:,:,t)*[0 0 0.20 1]';
%             tz(i+1)=text(z_axis_plus(1),z_axis_plus(2),z_axis_plus(3),...
%                 ['z',num2str(i)],'Color','r');
%             handax(i+1) = plot3([origin(1) x_axis(1)],...
%                 [origin(2) x_axis(2)],[origin(3) x_axis(3)],'r');
%             handay(i+1) = plot3([origin(1) y_axis(1)],...
%                 [origin(2) y_axis(2)],[origin(3) y_axis(3)],'r');
%             handaz(i+1) = plot3([origin(1) z_axis(1)],...
%                 [origin(2) z_axis(2)],[origin(3) z_axis(3)],'r');
% 
%             %END EFFECTOR 
% 
%             TCP = DHM_final(:,:,t)*[0 0 0 1]';
%             %Get one corner of the end effector
%             end_effector1 = DHM_final(:,:,t)*[0 -table(6,2)/2 -...
%                 table(6,2)/2 1]';
%             %Get ther other corner of the end effector
%             end_effector2 = DHM_final(:,:,t)*[0 table(6,2)/2 -...
%                 table(6,2)/2 1]'; 
%             %Get ther outer point of the end effector
%             end_effector3 = DHM_final(:,:,t)*[0 -table(6,2)/2 ...
%                 table(6,2)/2 1]';
%             %Get ther other outer point of the end effector
%             end_effector4 = DHM_final(:,:,t)*[0 table(6,2)/2 ...
%                 table(6,2)/2 1]'; 
% 
%             if t > 1
%             delete(handleTCP) %To only draw the next step
%             end
%             handleTCP = plot3([TCP(1) TCP(1)], [TCP(2) TCP(2)],...
%                 [0 TCP(3)],'k--'); %Plot the projection of the TCP to the
%             %fixed xy-plane
% 
%             %Draw the end effector. One plot for each line
%             if t > 1
%             delete(handleef1) %To only draw the next step
%             delete(handleef2) %To only draw the next step
%             delete(handleef3) %To only draw the next step
%             end
%             %Plot first segment of the end effector
%             handleef1 = plot3([end_effector1(1) end_effector3(1)],...
%                 [end_effector1(2) end_effector3(2)],...
%                 [end_effector1(3) end_effector3(3)]);  
%             %Plot second segment of the end effector
%             handleef2 = plot3([end_effector2(1) end_effector4(1)],...
%                 [end_effector2(2) end_effector4(2)],...
%                 [end_effector2(3) end_effector4(3)]);  
%             %Plot third segment of the end effector
%             handleef3 = plot3([end_effector1(1) end_effector2(1)],...
%                 [end_effector1(2) end_effector2(2)],...
%                 [end_effector1(3) end_effector2(3)]);  
%         end %if i == njoints
    end %for i
    axis equal;
    xlabel('x-axis');
    ylabel('y-axis');
    zlabel('z-axis');
    title('Plot of Robot with Joint Movements');
    pause(time);
end %for t

  
end %function

