% ********************************************************************
% Scrioipt simulating the MIP without hydrodynamics, controlled by LQG.
% ********************************************************************
%
% ***********************************
% Author: Rotem Sharoni
% Last updated: 2nd of May 2016
% ***********************************
%

x_init = [0 deg2rad(0) deg2rad(5) 0 0 0]'; %Initial "pose" of the MIP
rt = 0.135; % Radius from CG to motor axis in m
Tr=[1 1 1;
    0 0 0;
    0 rt -rt]; %Thrust allocation
pinvTr = pinv(Tr); % Pseudoinverse


%System:
g   = 9.81;   % gravitational accleration

%Vessel:
r = 0.548/2;  %Radius of te CS Saucer
h = 0.129;    %Total height of the CS Saucer
m_0 = 6.34;   % [kg],     vessel mass
I_0 = (1/12)*m_0*(3*r^2+h^2); %Mass moment of inertia about y_b axis (solid cylinder)
l_0 = 0.15;    % [m],      joint distance from center of pitch 
l_z = 0.15;   % [m] Arm from centre of pitch to thrusters

%Pendulum:
m_1 = 0.2;   % [kg],     pendulum mass
l_1 = 1.5;     % [m],      pendulum length 
I_1 = (m_1*(l_1)^2)/3; %Pendulum inertia

%Linearizing:
theta_0  = 0;   theta_1  = 0;
theta_0d = 0; theta_1d = 0;

D0 = [m_0+m_1              m_1*l_0*cos(theta_0)             m_1*l_1*cos(theta_1)
     m_1*l_0*cos(theta_0) I_0+m_1*l_0^2                    m_1*l_0*l_1*cos(theta_0-theta_1)
     m_1*l_1*cos(theta_1) m_1*l_0*l_1*cos(theta_0-theta_1) m_1*l_1^2+I_1                   ];
dG0 = [0        0         0;
       0   -m_1*g*l_0     0;
       0        0    -m_1*g*l_1];

H = [1;l_z;0];
 
% State space matrices:
A = [zeros(3) eye(3);
    -D0\dG0 zeros(3)];
B = [zeros(3,1);D0\H];
C = [eye(3) zeros(3)]; %Measuring all positions

% Checking for controllability:
Mc = ctrb(A,B); %Controllability matrix
Rc = rank(Mc);
Mo = obsv(A,C); %Observability matrix
Ro = rank(Mo);

% LQG:
n=6; %States in the system
%Controller:
Q = diag([1 1 700 1 1 100]); %Q matrix in the cost function
R = 100; % R in the cost function is a scalar in this case
[K_lqr,P,e] = lqr(A,B,Q,R,0); %Finding the feedback gain matrix K
Ref = zeros(6,1); %Reference signal (desired state)

%Observer
eigenvalues = eig(A-B*K_lqr); %Eigenvalues of the closed loop system
G = [1 1 0 1 1 0]'; %Process noise, not affecting pendulum
Ts = 0.05;
[Ad,Bd] = c2d(A,B,Ts);
[Ad,Gd] = c2d(A,G,Ts);
Cd = C;
Qkf = .1*eye(1); %Process noise
Rkf = eye(3)*1e-6; %Measurement noise

% Plotting and simulation: 
tsim = 10; %Simulation time, s
sim MIPv2_Kalman

%Plot the results:
set(groot, 'defaultAxesTickLabelInterpreter','latex'); 
set(groot, 'defaultLegendInterpreter','latex');

figure %Plot the positions
subplot(3,1,1)
plot(x.time,x.data(:,1))
hold on 
plot(x.time,x_hat.data(:,1));
legend('$x$','$\hat{x}$')
xlabel('Time (s)')
title('Position')
subplot(3,1,2)
plot(x.time,rad2deg(x.data(:,2)))
hold on 
plot(x.time,rad2deg(x_hat.data(:,2)));
legend('$\theta_0$','$\hat{\theta}_0$')
xlabel('Time (s)')
title('Pitch [deg]')
subplot(3,1,3)
plot(x.time,rad2deg(x.data(:,3)))
hold on 
plot(x.time,rad2deg(x_hat.data(:,3)));
legend('$\theta_1$','$\hat{\theta}_1$')
xlabel('Time (s)')
title('Pendulum angle [deg]')

figure %Plot the velocities
subplot(3,1,1)
plot(x.time,x.data(:,4))
hold on 
plot(x.time,x_hat.data(:,4));
legend('$\dot{x}$','$\dot{\hat{x}}$')
xlabel('Time (s)')
title('Velocity')
subplot(3,1,2)
plot(x.time,rad2deg(x.data(:,5)))
hold on 
plot(x.time,rad2deg(x_hat.data(:,5)));
legend('$\dot{\theta}_0$','$\dot{\hat{\theta}}_0$')
xlabel('Time (s)')
title('Pitch rate [deg/s]')
subplot(3,1,3)
plot(x.time,rad2deg(x.data(:,6)))
hold on 
plot(x.time,rad2deg(x_hat.data(:,6)));
legend('$\dot{\theta}_1$','$\dot{\hat{\theta}}_1$')
xlabel('Time (s)')
title('Pendulum angle rate [deg/s]')
