% ********************************************************************
% This script initializes parameters and runs the MIP model with surge and
% pitch coupling, as well as hydrodynamic forces. The model is controlled
% with an LQR.
% ********************************************************************
%
% ***********************************
% Author: Rotem Sharoni
% Last updated: 15th of April 2016
% ***********************************
%

% Initial parameters: 
x_init = [0 deg2rad(0) deg2rad(5) 0 0 0]'; %Initial "pose" of the MIP
rt = 0.135; % Radius from CG to motor axis in m
Tr=[1 1 1;
    0 0 0;
    0 rt -rt]; %Thrust allocation
pinvTr = pinv(Tr); % Pseudoinverse

% Parameters:
g = 9.81;     % gravitational accleration
rho = 1000;   % [kg/m^3], density of water

% Vessel data:
m_0 = 6.34;   % [kg],     vessel mass
l_0 = 0.15;    % [m],      joint distance from center of pitch 
r = 0.548/2;  % [m],      radius of te CS Saucer
l_z = 0.1; %Moment arm in pitch (m)
h = 0.129;    % [m]       total height of the CS Saucer
V = m_0/rho; % [m^3],    volume displacement = mass displacement/rho
I_y = (1/12)*m_0*(3*r^2+h^2); % [kgm^2], Mass moment of inertia about y axis
I_0 = I_y;
I_x = I_y;    % Cylindrical shape
% I_z = (m_0*r^2)/2; %[ kgm^2], mass moment of inertia about z axis
I_z = 0.116; %From Idland (2015)
I_g = diag([I_x I_y I_z]); % Inertia tensor
r_g = [0 0 0]'; % Location of CG wrt CO

% Pendulum data:
m_1 = 0.2;   % [kg],     pendulum mass
l_1 = 1.5;      % [m],      pendulum length 
I_1 = (m_1*(l_1)^2)/3; % [kg*m^2], pendulum inertia

% Added mass:
X_dotu = -3.50; %Added mass in surge due to surge motion
X_dotq = -0;    %Added mass in surge due to motion in pitch
M_dotu = -0;    %Added mass in pitch due to surge motion
M_dotq = -3;    %Added mass in pitch due to pitch motion

% Damping:
X_u = -1.96;    %Linear damping in surge due to surge motion
M_q = -3;       %Linear damping in pitch due to pitch motion
X_uu = -7.09;   %Nonlinear damping in surge due to surge motion
M_qq = -8;      %Nonlinear damping in pitch due to pitch motion

% Restoring:
I = (pi/4)*r^4; %[m^4], Area moment of inertia
KG = (.129/2);  %[m], KG for the Saucer. Assuming it is in the centre
KB  = 0.038/2;  %KB assuming it is in the centre of submerged volume
BM = I/V;       %BM_L = BM_T
BG = KG-KB;    
GM = BM-BG;     %GM (GM_L = GM_T)

%%%%%%%%%%%%%%%%%%%% LINEAR SYSTEM: %%%%%%%%%%%%%%%%%%%%%%%%%
% Linear state
theta_0 = 0; theta_1 = 0; x_d = 0; theta_0d = 0; theta_1d = 0;
% Hydroidynamic matrices
%Mass:
M_RB = [m_0*eye(3) -m_0*Smtrx(r_g);
        m_0*Smtrx(r_g) I_g];
M_A = -[X_dotu X_dotq;
        M_dotu M_dotq];
    
M = M_RB([1 5],[1 5]) + M_A; % 2 DOF mass     
M6 = zeros(6);      % Creating a 6 DOF mass matrix
M6([1 5],[1 5]) = M; %Inserting the relevant mass in the 6 by 6 matrix

%Restoring:
g_55 = rho*g*V*GM; % Fossen. p. 64

%Only linear damping:
Damp = -diag([X_u,M_q]); 

%Coriolis:
nu6 = zeros(6,1); %6 DOF velocity
Cor6 = m2c(M6,nu6);     % Fossen p.180 
Cor = Cor6([1 5],[1 5]);  %Relevant Corliolis, surge and pitch

% Assembling the matrices:
D0 = [-X_dotu+m_0+m_1,     -X_dotq+m_1*l_0*cos(theta_0),    m_1*l_1*cos(theta_1);
     -M_dotu+m_1*l_0*cos(theta_0),  -M_dotq+I_0+m_1*l_0^2,  m_1*l_0*l_1*cos(theta_0-theta_1);
     m_1*l_1*cos(theta_1),   m_1*l_0*l_1*cos(theta_0-theta_1),     m_1*l_1^2+I_1          ];
 
C = [Cor(1,1)+Damp(1,1), Cor(1,2)+Damp(1,2)-m_1*l_0*sin(theta_0)*theta_0d, -m_1*l_1*sin(theta_1)*theta_1d;
     Cor(2,1)+Damp(2,1),   Cor(2,2)+Damp(2,2),    m_1*l_0*l_1*sin(theta_0-theta_1)*theta_1d;
     0,     -m_1*l_0*l_1*sin(theta_0-theta_1)*theta_0d,         0                         ];
 
dG0 = [0 0 0;
     0 (g_55-m_1*g*l_0)*cos(theta_0) 0;
     0 0 -m_1*g*l_1*cos(theta_1)];
 
H = [1;l_z;0];
 
% State space matrices:
A = [zeros(3) eye(3);
    -D0\dG0 zeros(3)];
B = [zeros(3,1);D0\H];

% Checking for controllability:
Mc = ctrb(A,B); %Controllability matrix
Rc = rank(Mc);

% LQR:
Q = diag([1 1 7000 1 1 2000]); %Q matrix in the cost function
R = 1; % R in the cost function is a scalar in this case
[K,P,e] = lqr(A,B,Q,R,0); %Finding the feedback gain matrix K
Ref = zeros(6,1); %Reference signal (desired state)

% Plotting and simulation: 
tsim = 80; %Simulation time, s
sim MIPv2_hydro_LQR

%Plot the results:
set(groot, 'defaultAxesTickLabelInterpreter','latex'); 
set(groot, 'defaultLegendInterpreter','latex');

figure %Plot the state and velocities
subplot(3,1,1)
plot(x.time,x.data(:,1))
hold on 
plot(x.time,x.data(:,4));
legend('$x$','$\dot{x}$')
xlabel('Time (s)')
% title('Position')
subplot(3,1,2)
plot(x.time,rad2deg(x.data(:,2)))
hold on 
plot(x.time,rad2deg(x.data(:,5)));
legend('$\theta_0$','$\dot{\theta}_0$')
xlabel('Time (s)')
% title('Pitch [deg]')
subplot(3,1,3)
plot(x.time,rad2deg(x.data(:,3)))
hold on 
plot(x.time,rad2deg(x.data(:,6)));
legend('$\theta_1$','$\dot{\theta}_1$')
xlabel('Time (s)')
% title('Pendulum angle [deg]')

figure %Plot the forces and moments
subplot(3,1,1)
plot(tau.time,tau.data)
legend('X','M')
title('Thruster forces and moments')
xlabel('Time [s]')
ylabel('Force/Moment [N]/[Nm]')
subplot(3,1,2)
plot(Fhydro.time,Fhydro.data)
legend('"x"','"$\theta_0$"','"$\theta_1$"')
title('Hydrodynamic forces and moments')
xlabel('Time [s]')
ylabel('Force/Moment [N]/[Nm]')
subplot(3,1,3)
plot(Fpend.time,Fpend.data)
legend('"x"','"$\theta_0$"','"$\theta_1$"')
title('Pendulum forces and moments')
xlabel('Time [s]')
ylabel('Force/Moment [N]/[Nm]')

figure
plot(thrusterForces.time,thrusterForces.data)
title('Desired forces from each thruster')
legend('f1','f2','f3')
xlabel('Time [s]')
ylabel('Force [N]')
