function transfmat = transform_final(seq, type, values)

%*************************************************
% Homogeneous transformation matrices
% Author: Rotem Sharoni
% Last updated 20th of May 2015
%*************************************************
%
%*************************************************************************
% This function calculates the homogeneous transformation matrix for a
% series of movements. I.e, it calculates a matrix that can be used to
% translate points in ha mobile coordinate frame to a fixed frame.
%
% Input arguments:
%   -seq: a string that contains the sequence of movements that is
%           desired. The character 'i' indicates the position of the 
%           identity matrix.
%   -type: a boolean vector with the same length as 'seq' that indicates
%           the type of movement. Rotation=1 and Translation=0.
%   -values: a double precision vector (with the same length as 'seq' that 
%           contains the rotations (expressed in degrees) and 
%           translations (in meters) corresponding to the desired basic 
%           movements.
%
%  Example:
%  seq='xziyxz'; type=[0 1 1 1 0 1]; values=[1.5 30 0 -20 0.5 90];

% Output arguments:
%   -transfmat: a 4x4 homogeneous matrix that represents the transformation
%           that is equivalent to the sequence of movements.
%
%*************************************************************************

transfmat = eye(4); %Initialize final matrix


for i = 1:length(type)
    transfmat_step = eye(4); %Clean the step transfmat
    switch seq(i)
        case 'x'
            if type(i) == 0 %If translation
                transfmat_step(1,4) = values(i); 
            elseif type(i) == 1 %Rotation
                transfmat_step(1:3,1:3) = rotx(values(i)); 
            else
                error('The input "type" is not valid \n');
            end

        case 'y'
            if type(i) == 0 %If translation
                transfmat_step(2,4) = values(i); 
            elseif type(i) == 1 %Rotation
                transfmat_step(1:3,1:3) = roty(values(i)); 
            else
                error('The input "type" is not valid \n');
            end
                
        case 'z'
            if type(i) == 0 %If translation
                transfmat_step(3,4) = values(i); 
            elseif type(i) == 1 %Rotation
                transfmat_step(1:3,1:3) = rotz(values(i)); 
            else
                error('The input "type" is not valid \n');
            end

        otherwise 
            %Do nothing
    end %switch
    transfmat = transfmat*transfmat_step; %Multiplying each step
end %for

end %function